
//#include <sys/types.h>
//#include <sys/stat.h>
//#include <fcntl.h>

#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "utils.h"

#ifdef DMALLOC
#include <dmalloc.h>
#endif


/***************************************
 * Data
 ***************************************/


/***************************************
 *** Code                            ***
 ***************************************/

/***************************************
 * Queue Ops
 ***************************************/
int queue_insert(void *head, void *element) {
    QHead_t *h=head;
    QHead_t *e=element;

    if (h == NULL) return STS_FAILURE;
    if (e == NULL) return STS_FAILURE;

    if (e->next != 0) {
        fprintf(stderr, "WARN: queue_insert() - element->next != NULL\n");
        return STS_FAILURE;
    }

    e->next = h->next;
    h->next = e;

    return STS_SUCCESS;
}

void *queue_get_next(void *head) {
    QHead_t *h=head;
    if (h == NULL) return NULL;
    return h->next;
}

/***************************************
 * Common utils
 ***************************************/

/* 
 * open a file (usually in /proc or /sys) and fetch an INT value from it
 */
int get_int_from_file(char *filename, int *value) {
    FILE *f;
    int sts;

    f=fopen(filename, "r");
    if (!f) return STS_OPENERROR;

    sts = fscanf(f, "%i", value);
    if (sts <= 0) {
        fclose(f);
        return STS_FAILURE;
    }
    fclose(f);
    return STS_SUCCESS;
}


/* 
 * open a file (usually in /proc or /sys) and fetch a STRING value from it
 */
int get_string_from_file(char *filename, char *buffer, size_t buflen) {
    FILE *f;
    char *sts;

    f=fopen(filename, "r");
    if (!f) return STS_OPENERROR;

    sts = fgets(buffer, buflen, f);
    if (sts == NULL) {
        fclose(f);
        return STS_FAILURE;
    }
    fclose(f);
    return STS_SUCCESS;
}

/*
 * open a file and read a hex encoded mac from it
 * mac mus be a char [6] array.
 */
int get_mac_from_file(char *filename, char *mac) {
    FILE *f;
    char buffer[20];
    unsigned int ui[MACLEN];
    int len;
    int i;

    /* open file */
    f=fopen(filename, "r");
    if (f == NULL) return STS_FAILURE;

    while (f && fgets(buffer, sizeof(buffer), f)) {

        /* skip empty lines */
        if (buffer[0] == '\n') continue;

        // MAC len is hardcoded 6 octets, : separated
        len = sscanf(buffer, "%x:%x:%x:%x:%x:%x", 
                     &ui[0], &ui[1], &ui[2], &ui[3], &ui[4], &ui[5]);
        if (len == MACLEN) {
            for (i=0; i<MACLEN; i++) {
                mac[i]=(char)ui[i];
            }
        break;
        }
    } // while

    /* close file */
    if (f) { i=fclose(f); }

    if (len != MACLEN) return STS_FAILURE;
    return STS_SUCCESS;
}
